@@ -2,8 +2,11 @@
 # This file is part of Hy, which is free software licensed under the Expat
 # license. See the LICENSE.
 
-from inspect import getargspec, formatargspec
-from hy.models import replace_hy_obj, HyExpression, HySymbol
+from hy._compat import PY3
+import hy.inspect
+from hy.models import replace_hy_obj, HyExpression, HySymbol, wrap_value
+from hy.lex.parser import mangle
+from hy._compat import str_type
 
 from hy.errors import HyTypeError, HyMacroExpansionError
 
@@ -33,11 +36,11 @@ def macro(name):
     This function is called from the `defmacro` special form in the compiler.
 
     """
+    name = mangle(name)
     def _(fn):
         fn.__name__ = '({})'.format(name)
         try:
-            argspec = getargspec(fn)
-            fn._hy_macro_pass_compiler = argspec.keywords is not None
+            fn._hy_macro_pass_compiler = hy.inspect.has_kwargs(fn)
         except Exception:
             # An exception might be raised if fn has arguments with
             # names that are invalid in Python.
@@ -64,21 +67,23 @@ def tag(name):
 
     """
     def _(fn):
-        fn.__name__ = '#{}'.format(name)
+        _name = mangle('#{}'.format(name))
+        if not PY3:
+            _name = _name.encode('UTF-8')
+        fn.__name__ = _name
         module_name = fn.__module__
         if module_name.startswith("hy.core"):
             module_name = None
-        _hy_tag[module_name][name] = fn
+        _hy_tag[module_name][mangle(name)] = fn
 
         return fn
     return _
 
 
-def require(source_module, target_module,
-            all_macros=False, assignments={}, prefix=""):
+def require(source_module, target_module, assignments, prefix=""):
     """Load macros from `source_module` in the namespace of
-    `target_module`. `assignments` maps old names to new names, but is
-    ignored if `all_macros` is true. If `prefix` is nonempty, it is
+    `target_module`. `assignments` maps old names to new names, or
+    should be the string "ALL". If `prefix` is nonempty, it is
     prepended to the name of each imported macro. (This means you get
     macros named things like "mymacromodule.mymacro", which looks like
     an attribute of a module, although it's actually just a symbol
@@ -91,16 +96,18 @@ def require(source_module, target_module,
     seen_names = set()
     if prefix:
         prefix += "."
+    if assignments != "ALL":
+        assignments = {mangle(str_type(k)): v for k, v in assignments}
 
     for d in _hy_macros, _hy_tag:
         for name, macro in d[source_module].items():
             seen_names.add(name)
-            if all_macros:
-                d[target_module][prefix + name] = macro
+            if assignments == "ALL":
+                d[target_module][mangle(prefix + name)] = macro
             elif name in assignments:
-                d[target_module][prefix + assignments[name]] = macro
+                d[target_module][mangle(prefix + assignments[name])] = macro
 
-    if not all_macros:
+    if assignments != "ALL":
         unseen = frozenset(assignments.keys()).difference(seen_names)
         if unseen:
             raise ImportError("cannot require names: " + repr(list(unseen)))
@@ -136,9 +143,7 @@ def make_empty_fn_copy(fn):
         # can continue running. Unfortunately, the error message that might get
         # raised later on while expanding a macro might not make sense at all.
 
-        argspec = getargspec(fn)
-        formatted_args = formatargspec(*argspec)
-
+        formatted_args = hy.inspect.format_args(fn)
         fn_str = 'lambda {}: None'.format(
             formatted_args.lstrip('(').rstrip(')'))
         empty_fn = eval(fn_str)
@@ -151,65 +156,61 @@ def make_empty_fn_copy(fn):
     return empty_fn
 
 
-def macroexpand(tree, compiler):
+def macroexpand(tree, compiler, once=False):
     """Expand the toplevel macros for the `tree`.
 
     Load the macros from the given `module_name`, then expand the (top-level)
-    macros in `tree` until it stops changing.
+    macros in `tree` until we no longer can.
 
     """
     load_macros(compiler.module_name)
-    old = None
-    while old != tree:
-        old = tree
-        tree = macroexpand_1(tree, compiler)
-    return tree
+    while True:
 
+        if not isinstance(tree, HyExpression) or tree == []:
+            break
+
+        fn = tree[0]
+        if fn in ("quote", "quasiquote") or not isinstance(fn, HySymbol):
+            break
+
+        fn = mangle(fn)
+        m = _hy_macros[compiler.module_name].get(fn) or _hy_macros[None].get(fn)
+        if not m:
+            break
+
+        opts = {}
+        if m._hy_macro_pass_compiler:
+            opts['compiler'] = compiler
+
+        try:
+            m_copy = make_empty_fn_copy(m)
+            m_copy(compiler.module_name, *tree[1:], **opts)
+        except TypeError as e:
+            msg = "expanding `" + str(tree[0]) + "': "
+            msg += str(e).replace("<lambda>()", "", 1).strip()
+            raise HyMacroExpansionError(tree, msg)
+
+        try:
+            obj = m(compiler.module_name, *tree[1:], **opts)
+        except HyTypeError as e:
+            if e.expression is None:
+                e.expression = tree
+            raise
+        except Exception as e:
+            msg = "expanding `" + str(tree[0]) + "': " + repr(e)
+            raise HyMacroExpansionError(tree, msg)
+        tree = replace_hy_obj(obj, tree)
+
+        if once:
+            break
+
+    tree = wrap_value(tree)
+    return tree
 
 def macroexpand_1(tree, compiler):
     """Expand the toplevel macro from `tree` once, in the context of
-    `module_name`."""
-    if isinstance(tree, HyExpression):
-        if tree == []:
-            return tree
-
-        fn = tree[0]
-        if fn in ("quote", "quasiquote"):
-            return tree
-        ntree = HyExpression(tree[:])
-        ntree.replace(tree)
-
-        opts = {}
-
-        if isinstance(fn, HySymbol):
-            m = _hy_macros[compiler.module_name].get(fn)
-            if m is None:
-                m = _hy_macros[None].get(fn)
-            if m is not None:
-                if m._hy_macro_pass_compiler:
-                    opts['compiler'] = compiler
-
-                try:
-                    m_copy = make_empty_fn_copy(m)
-                    m_copy(compiler.module_name, *ntree[1:], **opts)
-                except TypeError as e:
-                    msg = "expanding `" + str(tree[0]) + "': "
-                    msg += str(e).replace("<lambda>()", "", 1).strip()
-                    raise HyMacroExpansionError(tree, msg)
-
-                try:
-                    obj = m(compiler.module_name, *ntree[1:], **opts)
-                except HyTypeError as e:
-                    if e.expression is None:
-                        e.expression = tree
-                    raise
-                except Exception as e:
-                    msg = "expanding `" + str(tree[0]) + "': " + repr(e)
-                    raise HyMacroExpansionError(tree, msg)
-                replace_hy_obj(obj, tree)
-                return obj
-        return ntree
-    return tree
+    `compiler`."""
+    return macroexpand(tree, compiler, once=True)
 
 
 def tag_macroexpand(tag, tree, compiler):
